@@ -18,8 +18,6 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <string>
-
 #include "mrn_mysql.h"
 
 #if MYSQL_VERSION_ID >= 50500
@@ -27,7 +25,6 @@
 #  include <sql_base.h>
 #endif
 #include "mrn_err.h"
-#include "mrn_sys.hpp"
 #include "mrn_table.hpp"
 #include "mrn_mysql_compat.h"
 #include <mrn_lock.hpp>
@@ -64,14 +61,16 @@ extern "C" {
 #endif
 
 extern HASH mrn_open_tables;
-extern pthread_mutex_t mrn_open_tables_mutex;
+extern mysql_mutex_t mrn_open_tables_mutex;
 extern HASH mrn_long_term_share;
-extern pthread_mutex_t mrn_long_term_share_mutex;
+extern mysql_mutex_t mrn_long_term_share_mutex;
 extern char *mrn_default_parser;
 extern char *mrn_default_wrapper_engine;
 extern handlerton *mrn_hton_ptr;
 extern HASH mrn_allocated_thds;
-extern pthread_mutex_t mrn_allocated_thds_mutex;
+extern mysql_mutex_t mrn_allocated_thds_mutex;
+extern PSI_mutex_key mrn_share_mutex_key;
+extern PSI_mutex_key mrn_long_term_share_auto_inc_mutex_key;
 
 static char *mrn_get_string_between_quote(const char *ptr)
 {
@@ -300,10 +299,11 @@ void mrn_get_partition_info(const char *table_name, uint table_name_length,
 
 int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
 {
-  int i, error;
+  int i, error = 0;
   int title_length;
   const char *sprit_ptr[2];
   const char *tmp_ptr, *start_ptr;
+  char *params_string = NULL;
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   partition_element *part_elem;
   partition_element *sub_elem;
@@ -370,8 +370,15 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
     }
 
     {
-      std::string params_string(params_string_value, params_string_length);
-      sprit_ptr[0] = params_string.c_str();
+      params_string = my_strndup(params_string_value,
+                                 params_string_length,
+                                 MYF(MY_WME));
+      if (!params_string) {
+        error = HA_ERR_OUT_OF_MEM;
+        goto error;
+      }
+
+      sprit_ptr[0] = params_string;
       while (sprit_ptr[0])
       {
         if ((sprit_ptr[1] = strchr(sprit_ptr[0], ',')))
@@ -405,6 +412,12 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
         case 6:
           MRN_PARAM_STR("engine", engine);
           break;
+        case 10:
+          MRN_PARAM_STR("normalizer", normalizer);
+          break;
+        case 13:
+          MRN_PARAM_STR("token_filters", token_filters);
+          break;
         case 17:
           MRN_PARAM_STR("default_tokenizer", default_tokenizer);
           break;
@@ -412,6 +425,9 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
           break;
         }
       }
+
+      my_free(params_string);
+      params_string = NULL;
     }
   }
 
@@ -441,7 +457,7 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
         !strncasecmp(share->engine, MRN_GROONGA_STR, MRN_GROONGA_LEN)
       )
     ) {
-      my_free(share->engine, MYF(0));
+      my_free(share->engine);
       share->engine = NULL;
       share->engine_length = 0;
     } else {
@@ -458,9 +474,9 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
     }
   }
 
-  DBUG_RETURN(0);
-
 error:
+  if (params_string)
+    my_free(params_string);
   DBUG_RETURN(error);
 }
 
@@ -486,7 +502,7 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
   if (key_info->comment.length == 0)
   {
     if (share->key_parser[i]) {
-      my_free(share->key_parser[i], MYF(0));
+      my_free(share->key_parser[i]);
     }
     if (
       !(share->key_parser[i] = my_strdup(mrn_default_parser, MYF(MY_WME)))
@@ -560,12 +576,12 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
   }
 
   if (param_string)
-    my_free(param_string, MYF(0));
+    my_free(param_string);
   DBUG_RETURN(0);
 
 error:
   if (param_string)
-    my_free(param_string, MYF(0));
+    my_free(param_string);
 #if MYSQL_VERSION_ID >= 50500
 error_alloc_param_string:
 #endif
@@ -663,12 +679,12 @@ int mrn_add_column_param(MRN_SHARE *share, Field *field, int i)
   }
 
   if (param_string)
-    my_free(param_string, MYF(0));
+    my_free(param_string);
   DBUG_RETURN(0);
 
 error:
   if (param_string)
-    my_free(param_string, MYF(0));
+    my_free(param_string);
 error_alloc_param_string:
   DBUG_RETURN(error);
 }
@@ -700,22 +716,26 @@ int mrn_free_share_alloc(
   uint i;
   MRN_DBUG_ENTER_FUNCTION();
   if (share->engine)
-    my_free(share->engine, MYF(0));
+    my_free(share->engine);
   if (share->default_tokenizer)
-    my_free(share->default_tokenizer, MYF(0));
+    my_free(share->default_tokenizer);
+  if (share->normalizer)
+    my_free(share->normalizer);
+  if (share->token_filters)
+    my_free(share->token_filters);
   for (i = 0; i < share->table_share->keys; i++)
   {
     if (share->index_table && share->index_table[i])
-      my_free(share->index_table[i], MYF(0));
+      my_free(share->index_table[i]);
     if (share->key_parser[i])
-      my_free(share->key_parser[i], MYF(0));
+      my_free(share->key_parser[i]);
   }
   for (i = 0; i < share->table_share->fields; i++)
   {
     if (share->col_flags && share->col_flags[i])
-      my_free(share->col_flags[i], MYF(0));
+      my_free(share->col_flags[i]);
     if (share->col_type && share->col_type[i])
-      my_free(share->col_type[i], MYF(0));
+      my_free(share->col_type[i]);
   }
   DBUG_RETURN(0);
 }
@@ -727,8 +747,8 @@ void mrn_free_long_term_share(MRN_LONG_TERM_SHARE *long_term_share)
     mrn::Lock lock(&mrn_long_term_share_mutex);
     my_hash_delete(&mrn_long_term_share, (uchar*) long_term_share);
   }
-  pthread_mutex_destroy(&long_term_share->auto_inc_mutex);
-  my_free(long_term_share, MYF(0));
+  mysql_mutex_destroy(&long_term_share->auto_inc_mutex);
+  my_free(long_term_share);
   DBUG_VOID_RETURN;
 }
 
@@ -757,8 +777,9 @@ MRN_LONG_TERM_SHARE *mrn_get_long_term_share(const char *table_name,
     long_term_share->table_name = tmp_name;
     long_term_share->table_name_length = table_name_length;
     memcpy(long_term_share->table_name, table_name, table_name_length);
-    if (pthread_mutex_init(&long_term_share->auto_inc_mutex,
-                           MY_MUTEX_INIT_FAST))
+    if (mysql_mutex_init(mrn_long_term_share_auto_inc_mutex_key,
+                         &long_term_share->auto_inc_mutex,
+                         MY_MUTEX_INIT_FAST) != 0)
     {
       *error = HA_ERR_OUT_OF_MEM;
       goto error_init_auto_inc_mutex;
@@ -772,9 +793,9 @@ MRN_LONG_TERM_SHARE *mrn_get_long_term_share(const char *table_name,
   DBUG_RETURN(long_term_share);
 
 error_hash_insert:
-  pthread_mutex_destroy(&long_term_share->auto_inc_mutex);
+  mysql_mutex_destroy(&long_term_share->auto_inc_mutex);
 error_init_auto_inc_mutex:
-  my_free(long_term_share, MYF(0));
+  my_free(long_term_share);
 error_alloc_long_term_share:
   DBUG_RETURN(NULL);
 }
@@ -894,7 +915,9 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
       share->wrap_table_share = wrap_table_share;
     }
 
-    if (pthread_mutex_init(&share->mutex, MY_MUTEX_INIT_FAST))
+    if (mysql_mutex_init(mrn_share_mutex_key,
+                         &share->mutex,
+                         MY_MUTEX_INIT_FAST) != 0)
     {
       *error = HA_ERR_OUT_OF_MEM;
       goto error_init_mutex;
@@ -916,11 +939,11 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
 
 error_hash_insert:
 error_get_long_term_share:
-  pthread_mutex_destroy(&share->mutex);
+  mysql_mutex_destroy(&share->mutex);
 error_init_mutex:
 error_parse_table_param:
   mrn_free_share_alloc(share);
-  my_free(share, MYF(0));
+  my_free(share);
 error_alloc_share:
   DBUG_RETURN(NULL);
 }
@@ -936,7 +959,7 @@ int mrn_free_share(MRN_SHARE *share)
       plugin_unlock(NULL, share->plugin);
     mrn_free_share_alloc(share);
     thr_lock_delete(&share->lock);
-    pthread_mutex_destroy(&share->mutex);
+    mysql_mutex_destroy(&share->mutex);
     if (share->wrapper_mode) {
 #ifdef MRN_TABLE_SHARE_HAVE_LOCK_SHARE
       mysql_mutex_destroy(&(share->wrap_table_share->LOCK_share));
@@ -945,7 +968,7 @@ int mrn_free_share(MRN_SHARE *share)
       mysql_mutex_destroy(&(share->wrap_table_share->LOCK_ha_data));
 #endif
     }
-    my_free(share, MYF(0));
+    my_free(share);
   }
   DBUG_RETURN(0);
 }
@@ -1113,11 +1136,11 @@ void mrn_clear_alter_share(THD *thd)
     slot_data->alter_create_info = NULL;
     slot_data->disable_keys_create_info = NULL;
     if (slot_data->alter_connect_string) {
-      my_free(slot_data->alter_connect_string, MYF(0));
+      my_free(slot_data->alter_connect_string);
       slot_data->alter_connect_string = NULL;
     }
     if (slot_data->alter_comment) {
-      my_free(slot_data->alter_comment, MYF(0));
+      my_free(slot_data->alter_comment);
       slot_data->alter_comment = NULL;
     }
   }
